#!/bin/bash

usage() {
	printf "Usage:\n"
	printf "$0 URL\n"
	printf "$0 [-l] FN\n"
	printf "\tURL\turl to Vertimus web page (must start with http://)\n"
	printf "\tFN\tfilename\n"
	printf "\t-l\twork offline\n"
	exit $1
}

# Information obtained from Vertimus page
DL_MODULE=
DL_BRANCH=
DL_USER=
DL_FILE_URL=

# Information obtained from filename
FN_MODULE=
FN_BRANCH=

# Assigned translator
WIKI_USER=
WIKI_FULLNAME=
USER_EMAIL=
USER_FULLNAME=

#---------------------------------------------------------

LL=sk
LANGUAGE=Slovak

#---------------------------------------------------------
GDATE=
GEGREP=
DOS2UNIX=
case `uname -s` in
	Linux)
		DOS2UNIX="dos2unix"
		GDATE=date
		GEGREP=egrep
		;;
	SunOS)
		DOS2UNIX="dos2unix -ascii"
		GDATE=/usr/gnu/bin/date
		GEGREP=gegrep
		;;
	*)
		echo "`uname -s` is not supported operating system" >&2
		exit 1
		;;
esac
	
#---------------------------------------------------------

test $# -lt 1 && printf "ERROR: Missing arguments!\n\n" && usage 2

LOCAL=
URL=
FILE=
if test "$1" != "-l" ; then
	echo "$1" | egrep "^http://l10n.gnome.org/vertimus/" > /dev/null
	if test $? -eq 0 ; then
		URL="$1"
		shift
	fi
else
	LOCAL=y
	shift
fi

if test -z "$URL" ; then
	test $# -ne 1 && printf "ERROR: Invalid arguments!\n\n" && usage 2
	FILE="$1"
	test ! -f "$FILE" && printf "ERROR: File '$FILE' not found!\n\n" && usage 2
fi

#---------------------------------------------------------

TMPDIR=/tmp/gnome
mkdir -p $TMPDIR


fatal() {
	echo "FAILED"
	printf "FATAL: $1\n" >&2
	exit 1
}

WARNS=0
warning() {
	echo "FAILED"
	printf "WARNING: $1\n" >&2
	WARNS=$(($WARNS+1))
}

NOTES=0
note() {
	printf "NOTE: $1\n" >&2
	NOTES=$(($NOTES+1))
}


vertimus() {
	echo "========"
	echo "Vertimus"
	echo "========"

	lynx $URL -dump -width=200 > $TMPDIR/vertimus
	UPLOAD=`cat $TMPDIR/vertimus | $GEGREP -B 2 -A 1 "^ *\[[0-9]*\]\[download.png\].*merged\.po$" | tail -4`
	printf "Check for PO file availability... "
	test `echo "$UPLOAD" | wc -l` -ne 4 && fatal "PO file is not available on Vertimus page"
	echo "ok"

	printf "Check for \"Upload the new translation\" action... "
	echo "$UPLOAD" | grep "Upload the new translation" > /dev/null
	if test $? -ne 0 ; then
		warning "No \"Upload the new translation\" action used for the file upload"
	else
		echo "ok"
	fi

	POT_LINK_NO=`cat $TMPDIR/vertimus | sed -e 's/^\(.*\)$/x\1/' | awk '$1 == "xActions" {exit} P==1 {print $0} $1 == "xState:" {P=1}' | egrep "^x   \[[0-9]+\]Download POT file" | sed -e 's/^x *\[\([0-9]*\)\].*$/\1/'`
	POT=`cat $TMPDIR/vertimus | awk '$1 == "References" {P=1} P==1 {print $0}' | egrep "^ *$POT_LINK_NO\. " | awk '{print $2}' | sed -e 's,^.*/\([^/]*\)$,\1,'`
	DL_MODULE=`echo $POT | sed -e 's/^\([^\.]*\)\.[^\.]*\.pot$/\1/'`
	echo "Module: $DL_MODULE"

	DL_BRANCH=`echo $POT | sed -e 's/^[^\.]*\.\([^\.]*\)\.pot$/\1/'`
	echo "Branch: $DL_BRANCH"

	USER_LN=`echo "$UPLOAD" | head -1 | sed -e 's/^ *Person \[\([0-9]*\)\].*$/\1/'`
	DL_USER=`cat $TMPDIR/vertimus | awk '$1 == "References" {P=1} P==1 {print $0}' | egrep "^ *$USER_LN\. " | awk '{print $2}' | sed -e 's,^http://l10n.gnome.org/users/\(.*\)/$,\1,'`
	echo "Username: $DL_USER"

	LN=`echo "$UPLOAD" | tail -2 | head -1 | sed -e 's/^ *\[\([0-9]*\)\]\[download.png\].*$/\1/'`
	DL_FILE_URL=`cat $TMPDIR/vertimus | awk '$1 == "References" {P=1} P==1 {print $0}' | egrep "^ *$LN\. " | awk '{print $2}'`
	echo "File URL: $DL_FILE_URL"

	FILENAME=$DL_MODULE.$DL_BRANCH.$LL.po
	printf "Downloading file and saving as $FILENAME... "
	wget -q "$DL_FILE_URL" -O $TMPDIR/$FILENAME
	echo "done"
}

if test -n "$URL" ; then
	vertimus
else
	FILENAME=`basename $FILE`
	cp -p $FILE $TMPDIR/$FILENAME > /dev/null 2>&1
fi

echo "========"
echo "Filename"
echo "========"

FN_MODULE=`echo "$FILENAME" | sed -e 's/^\([^\.]*\)\..*$/\1/'`
echo "Module: $FN_MODULE"
FN_BRANCH=`echo "$FILENAME" | sed -e 's/^[^\.]*\.\([^\.]*\)\..*$/\1/'`
echo "Branch: $FN_BRANCH"

if test -n "$DL_MODULE" ; then
	printf "Check for module name... "
	if test "$DL_MODULE" != "$FN_MODULE" ; then
		warning "Module name does not match"
	else
		echo "ok"
	fi
else
	note "Unable to check module name against Vertimus"
fi
if test -n "$DL_BRANCH" ; then
	printf "Check for module branch... "
	if test "$DL_BRANCH" != "$FN_BRANCH" ; then
		warning "Branch does not match"
	else
		echo "ok"
	fi
else
	note "Unable to check module branch against Vertimus"
fi

check_translator() {
	echo "==================="
	echo "Assigned Translator"
	echo "==================="

	wget -q "http://live.gnome.org/SlovakTranslation?action=raw" --user-agent="" -O - | $DOS2UNIX > $TMPDIR/wiki

	MOD=
	SUBM=$FN_MODULE
	while test "$SUBM" ; do
		MOD1="${SUBM%%-*}"
		SUB1="${SUBM#*-}"
		test "$MOD" && MOD="${MOD}-"
		MOD="${MOD}${MOD1}"
		test "$SUBM" = "$SUB1" && SUB1=
		SUBM="${SUB1}"
		RE_MOD=`echo $MOD | sed 's/\+/\\\+/g'`
		RE_SUBM=`echo $SUBM | sed 's/\+/\\\+/g'`
		test "$RE_SUBM" && RE_SUBM="/$RE_SUBM"
		TRANS=`cat $TMPDIR/wiki | egrep "^\|\| \[\[http://l10n\.gnome\.org/module/$RE_MOD/\|$RE_MOD$RE_SUBM\]\] \|\| \[\[http://l10n\.gnome\.org/users/[^/]*/\|.*\]\] \|\|$" \
			| sed -e 's,^|| .* || \(.*\) ||$,\1,'`
		test "$TRANS" && break
	done

	printf "Check for assigned translator... "
	if test -z "$TRANS" ; then
		warning "No assigned translator"
		return
	fi
	echo "ok"

	WIKI_USER=`echo $TRANS | sed -e 's,\[\[http://l10n\.gnome\.org/users/\([^/]*\)/|[^]]*\]\]$,\1,'`
	echo "Username: $WIKI_USER"
	WIKI_FULLNAME=`echo $TRANS | sed -e 's,\[\[http://l10n\.gnome\.org/users/[^/]*/|\([^]]*\)\]\]$,\1,'`
	echo "Full name: $WIKI_FULLNAME"

	lynx http://l10n.gnome.org/users/$WIKI_USER -dump > $TMPDIR/wiki-user
	USER_FULLNAME=`cat $TMPDIR/wiki-user | egrep "^ *\[14\]" | sed -e 's/^ *\[14\]//'`
	USER_EMAIL=`cat $TMPDIR/wiki-user | egrep "^ *\[15\]" | sed -e 's/^ *\[15\]//' | sed -e 's/ dot /./g' -e 's/ at /@/g'`
	echo "User full name: $USER_FULLNAME"
	echo "User e-mail: $USER_EMAIL"

	printf "Check for translator full name... "
	if test "$WIKI_FULLNAME" != "$USER_FULLNAME" ; then
		warning "Assigned translator full name differs from user's registration"
	else
		echo "ok"
	fi

	if test -n "$DL_USER" ; then
		printf "Check whether uploaded by assigned translator... "
		if test "$WIKI_USER" != "$DL_USER" ; then
			warning "Not uploaded by the assigned translator"
		else
			echo "ok"
		fi
	else
		note "Unable to check translator against Vertimus"
	fi
}

if test -z "$LOCAL" ; then
	check_translator
else
	note "Unable to check assigned translator (working offline)"
fi

echo "======="
echo "PO File"
echo "======="

printf "Check for header comment length... "
cat $TMPDIR/$FILENAME | head -5 | egrep -v "^#" > /dev/null
test $? -eq 0 && fatal "Too short or missing header comment block (at least 5 lines starting with # are required)"
echo "ok"

parse_years() {
	CY="$1"
	YL=
	S=B
	while test -n "$CY" ; do
		CHAR="${CY:0:1}"
		CY="${CY:1}"
		case "$CHAR$S" in
			[0-9]B)
				N="$CHAR"
				S=N
				;;
			[0-9]N)
				N="$N$CHAR"
				;;
			[0-9]H)
				N2="$CHAR"
				S=I
				;;
			[0-9]I)
				N2="$N2$CHAR"
				;;
			,N)
				YL="$YL${YL:+ }$N"
				S=C
				;;
			,I)
				test $N -ge $N2 && S=E && break
				for i in `seq $N $N2` ; do
					YL="$YL${YL:+ }$i"
				done
				S=C
				;;
			-N)
				S=H
				;;
			" C")
				S=B
				;;
			*)
				S=E
				break
				;;
		esac
	done
	case "$S" in
		N)
			YL="$YL${YL:+ }$N"
			;;
		I)
			if test $N -ge $N2 ; then
				S=E
			else
				for i in `seq $N $N2` ; do
					YL="$YL${YL:+ }$i"
				done
			fi
			;;
		*)
			S=E
			;;
	esac
	echo "$YL"
	test "$S" != "E"
	return $?
}

cat $TMPDIR/$FILENAME | (
	printf "Check for descriptive title... "
	read -r LINE
	if test "$LINE" != "# $LANGUAGE translation of $FN_MODULE." ; then
		warning "Invalid descriptive title\n\tcurrent:\t$LINE\n\tshould be:\t# $LANGUAGE translation of $FN_MODULE."
	else
		echo "ok"
	fi
	
	Y_CURRENT=`TZ=CET date +"%Y"`
	echo "Current year: $Y_CURRENT"

	printf "Check for copyright line... "
	read -r LINE
	echo "$LINE" | egrep "^# Copyright \(C\) .+ Free Software Foundation, Inc\.$" > /dev/null
	if test $? -ne 0 ; then
		warning "Invalid copyright line\n\tcurrent:\t$LINE\n\tshould be:\t# Copyright (C) YEAR(S) Free Software Foundation, Inc."
		COPYRIGHT_YEAR=
	else
		echo "ok"
		COPYRIGHT_YEAR=`echo "$LINE" | sed -e 's/^# Copyright (C) \(.*\) Free Software Foundation, Inc\.$/\1/'`
		echo "Copyright year(s): $COPYRIGHT_YEAR"

		printf "Parse copyright years... "
		parse_years "$COPYRIGHT_YEAR" > $TMPDIR/years
		if test $? -ne 0 ; then
			warning "Invalid copyright year(s) format"
		else
			echo "done"
			YL="`cat $TMPDIR/years`"
			echo "Copyright year(s) list: $YL"
			COPYRIGHT_YEAR_LIST=$YL

			while true ; do
				printf "Check for copyright year sequence... "
				Y_C=0
				for Y in $YL ; do
					if test $Y_C -ge $Y ; then
						warning "Error in sequence: $Y_C $Y"
						break 2
					fi
					if test $Y -lt 1999 ; then
						warning "Suspicious first copyright year (less than 1999): $Y"
						break 2
					fi
					Y_C=$Y
				done
				echo "ok"

				printf "Check for last copyright year... "
				if test $Y -ne $Y_CURRENT ; then
					warning "Last copyright year is different from the current one"
				else
					echo "ok"
				fi

				break
			done
		fi
	fi

	printf "Check for copyright note... "
	read -r LINE
	if test "$LINE" != "# This file is distributed under the same license as the $FN_MODULE package." ; then
		warning "Invalid copyright note\n\tcurrent:\t$LINE\n\tshould be:\t# This file is distributed under the same license as the $FN_MODULE package."
	else
		echo "ok"
	fi

	echo "Check for translators:"
	NUM=1
	YEAR_LAST=0
	YEAR_LIST=
	TRL_NAME=
	TRL_EMAIL=
	read -r LINE
	while true ; do
		printf "Check translator line #$NUM format... "
		echo "$LINE" | egrep "^# .* <[^<>]*>, .*\.$" > /dev/null
		if test $? -ne 0 ; then
			warning "Invalid translator line format\n\tcurrent:\t$LINE\n\tshould be:\t# NAME <EMAIL@ADDRESS>, YEAR(S)."
		else
			echo "ok"
			TRL_NAME=`echo "$LINE" | sed -e 's/^# \(.*\) <[^<>]*>, .*\.$/\1/'`
			echo "Translator #$NUM name: $TRL_NAME"
			TRL_EMAIL=`echo "$LINE" | sed -e 's/^# .* <\([^<>]*\)>, .*\.$/\1/'`
			echo "Translator #$NUM e-mail: $TRL_EMAIL"
			TRL_YEARS=`echo "$LINE" | sed -e 's/^# .* <[^<>]*>, \(.*\)\.$/\1/'`
			echo "Translator #$NUM years: $TRL_YEARS"

			printf "Translator #%d parse years... " $NUM
			parse_years "$TRL_YEARS" > $TMPDIR/years
			if test $? -ne 0 ; then
				warning "Invalid translator year(s) format"
			else
				echo "done"
				YL="`cat $TMPDIR/years`"
				echo "Translator #$NUM year(s) list: $YL"
				YEAR_LIST="$YEAR_LIST $YL"

				while true ; do
					printf "Translator #%d year sequence check... " $NUM
					Y_C=0
					for Y in $YL ; do
						if test $Y_C -ge $Y ; then
							warning "Error in sequence: $Y_C $Y"
							break 2
						fi
						if test $Y -lt 1999 ; then
							warning "Suspicious first translator year (less than 1999): $Y"
							break 2
						fi
						Y_C=$Y
					done
					echo "ok"

					printf "Translator #%d last year check... " $NUM
					if test $Y -lt $YEAR_LAST ; then
						warning "Last translator year is lesser than previous translator\n\tprevious:\t$YEAR_LAST\n\tcurrent:\t$Y"
					else
						echo "ok"
					fi
					YEAR_LAST=$Y

					break
				done
			fi
		fi
		read -r LINE
		test "$LINE" = "#" && break
		test "${LINE:0:1}" != "#" && break
		NUM=$(($NUM+1))
	done
	printf "Check for last translator year... "
	if test $YEAR_LAST -ne $Y_CURRENT ; then
		warning "Last translator year is different from the current one"
	else
		echo "ok"
	fi

	YEAR_LIST=`for Y in $YEAR_LIST ; do echo "$Y" ; done | sort -u`
	YEAR_LIST=`echo $YEAR_LIST`
	echo "Translator year list: $YEAR_LIST"

	printf "Check for copyright and translator year match... "
	if test "$COPYRIGHT_YEAR_LIST" != "$YEAR_LIST" ; then
		warning "Copyright and translator year list does not match\n\tcopyright years:\t$COPYRIGHT_YEAR_LIST\n\ttranslators years:\t$YEAR_LIST"
	else
		echo "ok"
	fi

	printf "Check for initial comment block termination... "
	while true ; do
		echo "$LINE" | egrep "^#" > /dev/null
		if test $? -ne 0 ; then
			warning "Initial comment block not terminated with '#'"
			break
		fi
		echo "$LINE" | egrep "^#$" > /dev/null
		if test $? -eq 0 ; then
			echo "ok"
			read -r LINE
			break
		fi
		read -r LINE
	done

	printf "Check for garbage in header comment... "
	echo "$LINE" | grep "^#" > /dev/null
	if test $? -eq 0 ; then
		GARBAGE="\t$LINE"
		while true ; do
			read -r LINE
			echo "$LINE" | grep "^#" > /dev/null
			test $? -ne 0 && break
			GARBAGE="$GARBAGE\n\t$LINE"
		done
		warning "Garbage in header comment:\n$GARBAGE"
	else
		echo "ok"
	fi

	printf "Check for msgid in first entry... "
	echo "$LINE" | egrep "^msgid \"\"$" > /dev/null
	if test $? -ne 0 ; then
		warning "Missing or invalid msgid line"
	else
		echo "ok"
	fi

	printf "Check for msgstr in first entry... "
	read -r LINE
	echo "$LINE" | egrep "^msgstr \"\"$" > /dev/null
	if test $? -ne 0 ; then
		warning "Missing or invalid msgstr line"
	else
		echo "ok"
	fi

	printf "Check for Project-Id-Version... "
	read -r LINE
	if test "$LINE" != "\"Project-Id-Version: $FN_MODULE\\n\"" ; then
		warning "Missing or invalid Project-Id-Version line\n\tcurrent:\t${LINE//\\/\\\\}\n\tshould be:\t\"Project-Id-Version: $FN_MODULE\\\\n\""
	else
		echo "ok"
	fi

	printf "Skip Report-Msgid-Bugs-To... "
	read -r LINE
	echo "$LINE" | egrep "^\"Report-Msgid-Bugs-To: " > /dev/null
	test $? -eq 0 && read -r LINE && read -r LINE
	echo "done"

	printf "Skip POT-Creation-Date... "
	echo "$LINE" | egrep "^\"POT-Creation-Date: " > /dev/null
	test $? -eq 0 && read -r LINE
	echo "done"

	printf "Check PO-Revision-Date format... "
	echo "$LINE" | egrep "^\"PO-Revision-Date: [0-9][0-9][0-9][0-9]-[01][0-9]-[0-3][0-9] [0-2][0-9]:[0-5][0-9]\+[0-9][0-9][0-9][0-9]\\\\n\"$" > /dev/null
	if test $? -ne 0 ; then
		warning "Missing or invalid PO-Revision-Date line\n\tcurrent:\t${LINE//\\/\\\\}\n\tshould be:\t\"PO-Revision-Date: YYYY-MM-DD HH:MM+ZZZZ\\\\n\""
	else
		DATE=`echo "$LINE" | sed -e 's/^"PO-Revision-Date: \(.*\)\\\\n"$/\1/'`
		CON_DATE=`TZ=CET $GDATE -d "$DATE" +"%Y-%m-%d %H:%M%z"`
		if test "$DATE" != "$CON_DATE" ; then
			warning "Invalid PO-Revision-Date\n\tcurrent:\t$DATE\n\tcorrect format:\t$CON_DATE"
		else
			echo "ok"
		fi
	fi

	printf "Check Last-Translator format... "
	read -r LINE
	echo "$LINE" | egrep "^\"Last-Translator: .* <[^<>]*>\\\\n\"$" > /dev/null
	if test $? -ne 0 ; then
		warning "Invalid Last-Translator line format\n\tcurrent:\t${LINE//\\/\\\\}\n\tshould be:\t\"Last-Translator: NAME <EMAIL@ADDRESS>\\\\n\""
	else
		echo "ok"

		PO_NAME=`echo "$LINE" | sed -e 's/^"Last-Translator: \(.*\) <[^<>]*>\\\\n"$/\1/'`
		PO_EMAIL=`echo "$LINE" | sed -e 's/^"Last-Translator: .* <\([^<>]*\)>\\\\n"$/\1/'`
		echo "Last-Translator name: $PO_NAME"
		echo "Last-Translator e-mail: $PO_EMAIL"

		printf "Check Last-Translator full name against the header comments... "
		if test "$PO_NAME" != "$TRL_NAME" ; then
			warning "Fullname in Last-Translator differs from header comments\n\theader comments:\t$TRL_NAME\n\tLast-Translator:\t$PO_NAME"
		else
			echo "ok"
		fi

		printf "Check Last-Translator e-mail against the header comments... "
		if test "$PO_EMAIL" != "$TRL_EMAIL" ; then
			warning "E-mail in Last-Translator differs from header comments\n\theader comments:\t$TRL_EMAIL\n\tLast-Translator:\t$PO_EMAIL"
		else
			echo "ok"
		fi


		if test -n "$USER_FULLNAME" -a -n "$USER_EMAIL" ; then
			printf "Check Last-Translator name against assigned translator full name... "
			if test "$PO_NAME" != "$USER_FULLNAME" ; then
				warning "Last-Translator name differs from assigned translator full name\n\tassigned translator:\t$USER_FULLNAME\n\tLast-Translator:\t$PO_NAME"
			else
				echo "ok"
			fi

			printf "Check Last-Translator e-mail against assigned translator e-mail... "
			if test "$PO_EMAIL" != "$USER_EMAIL" ; then
				warning "Last-Translator e-mail differs from assigned translator e-mail\n\tassigned translator:\t$USER_EMAIL\n\tLast-Translator:\t$PO_EMAIL"
			else
				echo "ok"
			fi
		else
			note "Unable to check whether Last-Translator is assigned translator"
		fi
	fi

	printf "Check for Language-Team... "
	read -r LINE
	if test "$LINE" != "\"Language-Team: Slovak <gnome-sk-list@gnome.org>\\n\"" ; then
		warning "Invalid Language-Team\n\tcurrent:\t${LINE//\\/\\\\}\n\tshould be:\t\"Language-Team: Slovak <gnome-sk-list@gnome.org>\\\\n\""
	else
		echo "ok"
	fi

	printf "Check for MIME-Version... "
	read -r LINE
	if test "$LINE" != "\"MIME-Version: 1.0\\n\"" ; then
		warning "Invalid MIME-Version\n\tcurrent:\t${LINE//\\/\\\\}\n\tshould be:\t\"MIME-Version: 1.0\\\\n\""
	else
		echo "ok"
	fi

	printf "Check for Content-Type... "
	read -r LINE
	if test "$LINE" != "\"Content-Type: text/plain; charset=UTF-8\\n\"" ; then
		warning "Invalid Content-Type\n\tcurrent:\t${LINE//\\/\\\\}\n\tshould be:\t\"Content-Type: text/plain; charset=UTF-8\\\\n\""
	else
		echo "ok"
	fi

	printf "Check for Content-Transfer-Encoding... "
	read -r LINE
	if test "$LINE" != "\"Content-Transfer-Encoding: 8bit\\n\"" ; then
		warning "Invalid Content-Transfer-Encoding\n\tcurrent:\t${LINE//\\/\\\\}\n\tshould be:\t\"Content-Transfer-Encoding: 8bit\\\\n\""
	else
		echo "ok"
	fi

	printf "Check for Plural-Forms... "
	read -r LINE
	echo "$LINE" | egrep "Plural-Forms" > /dev/null
	if test $? -eq 0 ; then
		if test "$LINE" != "\"Plural-Forms: nplurals=3; plural=(n==1) ? 1 : (n>=2 && n<=4) ? 2 : 0;\\n\"" ; then
			warning "Invalid Plural-Forms\n\tcurrent:\t${LINE//\\/\\\\}\n\tshould be:\t\"Plural-Forms: nplurals=3; plural=(n==1) ? 1 : (n>=2 && n<=4) ? 2 : 0;\\\\n\""
		else
			echo "ok"
		fi
	else
		echo "missing"
	fi

	echo "$WARNS" > $TMPDIR/warns
	echo "$NOTES" > $TMPDIR/notes
)
WARNS=`cat $TMPDIR/warns`
NOTES=`cat $TMPDIR/notes`

#---------------------------------------------------------

if test $WARNS -ne 0 ; then
	echo "" >&2
	echo "Total: $WARNS warninig(s) found (notes: $NOTES)" >&2
else
	echo ""
	echo "No problems found (notes: $NOTES)!"
fi
